//
// Created by JiangJian on 2018/9/7.
// Copyright (c) 2018 aliyun. All rights reserved.
//

#import <CoreBluetooth/CoreBluetooth.h>
#import "LPBSBreezeConnection.h"
#import "LPBSBreezeDevice.h"
#import "LKBRZLog.h"
#import "LKBreezeWraper.h"
#import "LKBreezeWraper+Inner.h"
#import "LKBRZTlvHelper.h"
#import "LKBRZConstant.h"
#import "NSData+DataToHex.h"

@interface LPBSBreezeConnection ()<LKBRZStateDelagete>

@property(nonatomic, strong) LKBreezeWraper *breezeWraper; //仅用作连接设备，收发数据
@property(nonatomic, strong) LPBSBreezeDevice *currentdeviceInfo;
@end

@implementation LPBSBreezeConnection

@synthesize connectionDelegate;
@synthesize connectionState;

- (instancetype)initWithDeviceInfo:(id<LPBSDiscoveryDevice>)deviceInfo {
    self = [super init];
    if (self) {
        _breezeWraper = [LKBreezeWraper new];
        _currentdeviceInfo = deviceInfo;
    }
    return self;
}

- (instancetype)initWithDeviceInfo:(id<LPBSDiscoveryDevice>)deviceInfo breeze:(LKBreezeWraper *)breezeWraper {
    self = [super init];
    if (self) {
        _breezeWraper = breezeWraper;
        _currentdeviceInfo = deviceInfo;
    }
    return self;
}

#pragma mark LPBSConnection
- (void)startConnect:(NSData *)authInfo result:(LPBSBoolResultBlockExtra)resultBlock {
    if(self.currentdeviceInfo != nil) {
        self.breezeWraper.stateDelagete = self;
        NSString *mac = self.currentdeviceInfo.extraInfo[@"mac"];
        [self.breezeWraper connectPeripheralUseMac:mac handler:^(NSError *error) {
            if(error != nil) {
                resultBlock(NO, error, nil);
                return;
            }
            //获取ProductKey，DeviceName，Sign
            [self.breezeWraper sendSystemData:[self genDeviceInfoCmd] responseHandler:^(NSData *data, NSError *error) {
                NSMutableDictionary *ret = nil;
                if(error == nil) {
                    ret = [NSMutableDictionary new];
                    [ret setObject:@"sha256" forKey:@"signMethod"];
                    [ret addEntriesFromDictionary:[self parseDeviceData:data]];
                    
                    NSDictionary *dic = [self.breezeWraper getConnectedPerpheralDeviceData];
                    [ret setObject:[dic objectForKey:@"internelProductId"] forKey:@"clientId"];
                }
                resultBlock(error == nil, error, ret);
            }];
        }];
    }
}

- (NSData *)genDeviceInfoCmd {
    //NSData *d1 = [LKBRZTlvHelper genTlvData:LKIoTBleSysCmdVersion data:nil];//version
    NSData *d2 = [LKBRZTlvHelper genTlvData:LKIoTBleSysCmdProductKey data:nil];//productKey
    NSData *d3 = [LKBRZTlvHelper genTlvData:LKIoTBleSysCmdDeviceName data:nil];//deviceName
    NSData *d4 = [LKBRZTlvHelper genTlvData:LKIoTBleSysCmdRandom data:nil];//random
    NSData *d5 = [LKBRZTlvHelper genTlvData:LKIoTBleSysCmdSign data:nil];//sign
    NSMutableData *data = [NSMutableData new];
    //[data appendData:d1];
    [data appendData:d2];
    [data appendData:d3];
    [data appendData:d4];
    [data appendData:d5];
    return data;
}

- (NSDictionary *)parseDeviceData:(NSData *)data {
    NSDictionary *dic = [LKBRZTlvHelper parseTlvData:data];
    NSString *productKey = [[NSString alloc] initWithData:[dic objectForKey:@(LKIoTBleSysCmdProductKey)] encoding:NSUTF8StringEncoding];
    NSString *deviceName = [[NSString alloc] initWithData:[dic objectForKey:@(LKIoTBleSysCmdDeviceName)] encoding:NSUTF8StringEncoding];
    NSString *sign = [[dic objectForKey:@(LKIoTBleSysCmdSign)] brz_dataToHex];
    
    NSMutableDictionary *deviceInfo = [NSMutableDictionary new];
    if(productKey != nil && deviceName != nil && sign != nil) {
        [deviceInfo setObject:productKey forKey:@"productKey"];
        [deviceInfo setObject:deviceName forKey:@"deviceName"];
        [deviceInfo setObject:sign forKey:@"sign"];
        return deviceInfo;
    } else {
        BRZLogError(@"parseDeviceData  error: %@", dic);
        return nil;
    }
}

- (void)stopConnect {
    [self.breezeWraper disconnectPeripheral:^(NSError *error) {
        self.breezeWraper.stateDelagete = nil;
    }];
}

- (void)asyncSendRequest:(NSData *)payload options:(NSDictionary *)options result:(LPBSBizRespCallback)resultCallback {
    [self.breezeWraper sendData:payload responseHandler:^(NSData *data, NSError *error) {
        LPBSBizResponse *rsp = [LPBSBizResponse new];
        rsp.dataObject = data;
        rsp.responseError = error;
        rsp.success = error == nil;
        resultCallback(rsp);
    }];
}

- (void)subscribeEvents:(LPBSBoolResultBlock)resultBlock {
    //无需实现
    resultBlock(YES, nil);
}

- (void)unsubscribeEvents:(LPBSBoolResultBlock)resultBlock {
    //无需实现
    resultBlock(YES, nil);
}

#pragma mark BRZStateDelagete
- (void)didDiscoverDevice:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI extra:(NSDictionary *)extra {
    //LPBSBreezeConnection不会使用到
    BRZLogError(@"!SHOULD NEVER OCCUR! didDiscoverDevice: %@", extra);
}

- (void)didReceivedData:(NSData *)data {
    BRZLogDebug(@"didReceivedData: %@", data);
    if(self.connectionDelegate != nil) {
        [self.connectionDelegate didReceiveEvent:data];
    }
}

- (void)didStateChanged:(NSUInteger)state {
    BRZLogDebug(@"didStateChanged: %d", state);
    NSUInteger lpbsState = LPBSConnectionDisconnected;
    switch (state) {
        case CBPeripheralStateDisconnected:
            lpbsState = LPBSConnectionDisconnected;
            break;
        case CBPeripheralStateConnecting:
            lpbsState = LPBSConnectionConnecting;
            break;
        case CBPeripheralStateConnected:
            lpbsState = LPBSConnectionConnected;;
            break;
        case CBPeripheralStateDisconnecting:
            lpbsState = LPBSConnectionDisconnecting;
            break;
        default:
            break;
    }
    self.connectionState = lpbsState;
    if(self.connectionDelegate != nil) {
        [self.connectionDelegate onConnectState:lpbsState];
    }
}

@end
